@@ -13,13 +13,19 @@ import math
 import numpy as np
 from scipy.io import readsav
 from astropy import wcs
+from astropy.io import ascii
+from astropy.table import Table
 from reproject import reproject_interp
 import subprocess as SP
 
 ## Local
-from iolib import fclean, read_fits, write_fits, read_csv, write_csv, read_ascii
-from alib import fixwcs, get_pc
-from mlib import nanavg, closest, bsplinterpol
+from iolib import (fitsext, csvext, ascext, fclean,
+                   read_fits, write_fits,
+                   # read_csv, write_csv, read_ascii,
+)
+from arrlib import allist, closest
+from mathlib import nanavg, bsplinterpol
+from astrolib import fixwcs, get_pc
 
 savext = '.sav'
 
@@ -35,7 +41,7 @@ class improve:
     '''
     def __init__(self, filIN, wmod=0, verbose=False):
         '''
-        self: filIN, wmod, hdr, w, cdelt, pc, cd, dim, Nx, Ny, Nw, im, wvl
+        self: filIN, wmod, hdr, w, cdelt, pc, cd, Ndim, Nx, Ny, Nw, im, wvl
         '''
         
         ## INPUTS
@@ -62,12 +68,12 @@ class improve:
         ds = read_fits(filIN)
         self.im = ds.data
         self.wvl = ds.wave
-        self.dim = len(self.im.shape)
-        if len(self.im.shape)==3:
+        self.Ndim = self.im.ndim
+        if self.Ndim==3:
             if self.im.shape[0]==1:
-                self.dim = 2 # Nw=1 patch
-        if self.dim==3:
-            self.Nw = len(self.wvl)
+                self.Ndim = 2 # Nw=1 patch
+            else:
+                self.Nw = len(self.wvl)
 
     def rand_norm(self, file=None, unc=None, sigma=1., mu=0.):
         '''
@@ -103,14 +109,14 @@ class improve:
             peak = 1/(1+tau)
             theta = np.random.normal(mu, sigma, self.im.shape) # ~N(0,1)
             flag = np.random.random(self.im.shape) # ~U(0,1)
-            if self.dim==2:
+            if self.Ndim==2:
                 for x in range(self.Nx):
                     for y in range(self.Ny):
                         if flag[y,x]<peak[y,x]:
                             self.im[y,x] += -abs(theta[y,x]) * unc[0][y,x]
                         else:
                             self.im[y,x] += abs(theta[y,x]) * unc[1][y,x]
-            elif self.dim==3:
+            elif self.Ndim==3:
                 for x in range(self.Nx):
                     for y in range(self.Ny):
                         for k in range(self.Nw):
@@ -126,7 +132,7 @@ class improve:
     def slice(self, filSL, postfix=''):
         ## 3D cube slicing
         slist = []
-        if self.dim==3:
+        if self.Ndim==3:
             hdr = self.hdr.copy()
             for kw in self.hdr.keys():
                 if '3' in kw:
@@ -150,7 +156,7 @@ class improve:
         ## Inversed square cube slicing
         inv_sq = 1./self.im**2
         slist = []
-        if self.dim==3:
+        if self.Ndim==3:
             hdr = self.hdr.copy()
             for kw in self.hdr.keys():
                 if '3' in kw:
@@ -234,11 +240,11 @@ class improve:
         ## OUTPUTS
         ##---------
         ## New image
-        if self.dim==3:
+        if self.Ndim==3:
             self.im = self.im[:, ymin:ymax, xmin:xmax] # gauss_noise inclu
             ## recover 3D non-reduced header
             self.hdr = read_fits(self.filIN).header
-        elif self.dim==2:
+        elif self.Ndim==2:
             self.im = self.im[ymin:ymax, xmin:xmax] # gauss_noise inclu
         ## Modify header
         ## Suppose no non-linear distortion
@@ -269,7 +275,7 @@ class islice(improve):
     postfix             postfix of output slice names
     ------ OUTPUT ------
     self: slist, path_tmp, 
-          (filIN, wmod, hdr, w, cdelt, pc, cd, dim, Nx, Ny, Nw, im, wvl)
+          (filIN, wmod, hdr, w, cdelt, pc, cd, Ndim, Nx, Ny, Nw, im, wvl)
     '''
     def __init__(self, filIN, filSL=None, filUNC=None, dist='norm', \
         slicetype=None, postfix=''):
@@ -353,7 +359,7 @@ class imontage(improve):
         fmod='ref', ext_pix=0, tmpdir=None):
         '''
         self: hdr_ref, path_tmp, 
-        (filIN, wmod, hdr, w, dim, Nx, Ny, Nw, im, wvl)
+        (filIN, wmod, hdr, w, Ndim, Nx, Ny, Nw, im, wvl)
         '''
         ## Set path of tmp files
         if tmpdir is None:
@@ -526,10 +532,10 @@ class imontage(improve):
             # phdu = fits.PrimaryHDU(header=hdr, data=self.im[k,:,:])
             # im_rep = reproject_interp(phdu, self.hdr_ref)[0]
         for s in self.slist:
-            im_rep = reproject_interp(s+'.fits', self.hdr_ref)[0]
+            im_rep = reproject_interp(s+fitsext, self.hdr_ref)[0]
             cube_rep.append(im_rep)
             write_fits(s+'rep_', self.hdr_ref, im_rep)
-            fclean(s+'.fits')
+            fclean(s+fitsext)
         self.im = np.array(cube_rep)
 
         comment = "Reprojected by <imontage>. "
@@ -706,7 +712,7 @@ class iswarp(improve):
                  verbose=False, tmpdir=None):
         '''
         self: path_tmp, verbose
-        (filIN, wmod, hdr, w, dim, Nx, Ny, Nw, im, wvl)
+        (filIN, wmod, hdr, w, Ndim, Nx, Ny, Nw, im, wvl)
         '''
         if verbose==False:
             devnull = open(os.devnull, 'w')
@@ -734,10 +740,7 @@ class iswarp(improve):
                 self.refheader = refheader
         else:
             ## Input files in list format
-            if isinstance(filIN, str):
-                filIN = [filIN]
-            else:
-                filIN = list(filIN)
+            filIN = allist(filIN)
             
             ## Images
             image_files = ' '
@@ -745,12 +748,12 @@ class iswarp(improve):
                 image = read_fits(filIN[i]).data
                 hdr = fixwcs(filIN[i]).header
                 file_ref = filIN[i]
-                if len(image.shape)==3:
+                if image.ndim==3:
                     ## Extract 1st frame of the cube
                     file_ref = path_tmp+os.path.basename(filIN[i])+'_ref'
                     write_fits(file_ref, hdr, image[0])
                 
-                image_files += file_ref+'.fits ' # input str for SWarp
+                image_files += file_ref+fitsext+' ' # input str for SWarp
             
             ## Create config file
             SP.call('swarp -d > swarp.cfg', \
@@ -824,10 +827,7 @@ class iswarp(improve):
             os.makedirs(path_comb)
 
         ## Input files in list format
-        if isinstance(file, str):
-            file = [file]
-        else:
-            file = list(file)
+        file = allist(file)
         
         ## Header
         ##--------
@@ -875,10 +875,10 @@ class iswarp(improve):
         for k in trange(Nw, leave=False, 
             desc='<iswarp> Combining (by wvl)'):
             for i in range(Nf):
-                image_files[k] += imlist[i][k]+'.fits '
+                image_files[k] += imlist[i][k]+fitsext+' '
 
                 if combtype=='wgt_avg':
-                    weight_files[k] += wgtlist[i][k]+'.fits '
+                    weight_files[k] += wgtlist[i][k]+fitsext+' '
 
             ## Create config file
             SP.call('swarp -d > swarp.cfg', \
@@ -998,10 +998,8 @@ class iconvolve(improve):
             self.rand_splitnorm(filUNC)
 
         ## Input kernel file in list format
-        if isinstance(kfile, str):
-            self.kfile = [kfile]
-        else:
-            self.kfile = list(kfile)
+        self.kfile = allist(kfile)
+
         ## doc (csv) file of kernel list
         self.klist = klist
         self.path_conv = convdir
@@ -1041,24 +1039,42 @@ class iconvolve(improve):
         sigma_per = fwhm_per / (2. * np.sqrt(2.*np.log(2.)))
         self.sigma_lam = np.sqrt(sigma_par * sigma_per)
         
+    # def choker(self, file):
+    #     ## CHOose KERnel(s)
+    #     lst = []
+    #     for i, image in enumerate(file):
+    #         ## check PSF profil (or is not a cube)
+    #         if self.sigma_lam is not None:
+    #             image = file[i]
+    #             ind = closest(self.psf, self.sigma_lam[i])
+    #             kernel = self.kfile[ind]
+    #         else:
+    #             image = file[0]
+    #             kernel = self.kfile[0]
+    #         ## lst line elements: image, kernel
+    #         k = [image, kernel]
+    #         lst.append(k)
+
+    #     ## write csv file
+    #     write_csv(self.klist, header=['Images', 'Kernels'], dset=lst)
+
     def choker(self, file):
         ## CHOose KERnel(s)
-        lst = []
-        for i, image in enumerate(file):
+        image = []
+        kernel = []
+        for i, filim in enumerate(file):
             ## check PSF profil (or is not a cube)
             if self.sigma_lam is not None:
-                image = file[i]
+                image.append(filim)
                 ind = closest(self.psf, self.sigma_lam[i])
-                kernel = self.kfile[ind]
+                kernel.append(self.kfile[ind])
             else:
-                image = file[0]
-                kernel = self.kfile[0]
-            ## lst line elements: image, kernel
-            k = [image, kernel]
-            lst.append(k)
+                image.append(file[0])
+                kernel.append(self.kfile[0])
 
         ## write csv file
-        write_csv(self.klist, header=['Images', 'Kernels'], dataset=lst)
+        dataset = Table([image, kernel], names=['Images', 'Kernels'])
+        ascii.write(dataset, self.klist+csvext, format='csv')
 
     def do_conv(self, idldir, verbose=False):
         '''
@@ -1073,7 +1089,7 @@ class iconvolve(improve):
 
         filename = os.path.basename(self.filIN)
 
-        if self.dim==3:
+        if self.Ndim==3:
             if self.path_conv is not None:
                 f2conv = self.slice(self.path_conv+filename) # gauss_noise inclu
             else:
@@ -1091,7 +1107,7 @@ class iconvolve(improve):
 
         ## OUTPUTS
         ##---------
-        if self.dim==3:
+        if self.Ndim==3:
             im = []
             self.slist = []
             for f in f2conv:
@@ -1207,7 +1223,8 @@ class sextract(improve):
         for i in range(Ny):
             ## Ny/Nsub should be integer, or there will be shift
             ispec = math.floor(i / (math.ceil(Ny/Nsub)))
-            spec_arr.append(read_ascii(self.path+'Y12spec'+str(ispec), '.spc', float))
+            # spec_arr.append(read_ascii(self.path+'Y12spec'+str(ispec), '.spc', float))
+            spec_arr.append(ascii.read(self.path+'Y12spec'+str(ispec)+'.spc'))
         spec_arr = np.array(spec_arr)
         Nw = len(spec_arr[0,:,0])
         
@@ -1226,7 +1243,7 @@ class sextract(improve):
                     unc_P[k][j][i] = (spec_arr[j,k,3]-spec_arr[j,k,1])
             wave[k] = spec_arr[0,k,0]
 
-        ## Save spec in wave ascending order
+        ## Save spec in wave ascending order$
         self.cube = cube[::-1]
         self.unc = unc[::-1]
         self.unc_N = unc_N[::-1]
@@ -1327,7 +1344,8 @@ def wclean(filIN, cmod='eq', cfile=None, \
     
     ind = [] # list of indices of wvl to remove
     if cfile is not None:
-        indarxiv = read_csv(cfile, 'Ind')[0]
+        # indarxiv = read_csv(cfile, 'Ind')[0]
+        indarxiv = ascii.read(cfile+csvext)['Ind']
         ind = []
         for i in indarxiv:
             ind.append(int(i))
@@ -1440,7 +1458,7 @@ def wclean(filIN, cmod='eq', cfile=None, \
         for i in ind:
             wlist.append([i, wave[i]])
         write_csv(filOUT+'_wclean_info', \
-            header=['Ind', 'Wavelengths'], dataset=wlist)
+            header=['Ind', 'Wavelengths'], dset=wlist)
 
     return data_new, wave_new
 
